define(['exports', 'react', 'lodash', 'react-fast-compare'], function (exports, React, lodash, isEqual) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
  var isEqual__default = /*#__PURE__*/_interopDefaultLegacy(isEqual);

  var useMount = function (fn) {
      React.useEffect(function () {
          fn();
      }, []);
  };

  var useUnmount = function (fn) {
      var fnRef = React.useRef(fn);
      // 如果 fn 的依赖更新了，就要更新卸载要执行的方法
      fnRef.current = fn;
      React.useEffect(function () {
          return function () {
              fnRef.current();
          };
      }, []);
  };

  var useEffectOnce = function (mountFn, unMountFn) {
      if (lodash.isFunction(mountFn)) {
          useMount(mountFn);
      }
      else {
          throw Error('mount need to receive a function');
      }
      if (lodash.isFunction(unMountFn)) {
          useUnmount(unMountFn);
      }
      else {
          throw Error('unMount need to receive a function');
      }
  };

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */

  function __read(o, n) {
      var m = typeof Symbol === "function" && o[Symbol.iterator];
      if (!m) return o;
      var i = m.call(o), r, ar = [], e;
      try {
          while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
      }
      catch (error) { e = { error: error }; }
      finally {
          try {
              if (r && !r.done && (m = i["return"])) m.call(i);
          }
          finally { if (e) throw e.error; }
      }
      return ar;
  }

  function __spreadArray(to, from, pack) {
      if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
          if (ar || !(i in from)) {
              if (!ar) ar = Array.prototype.slice.call(from, 0, i);
              ar[i] = from[i];
          }
      }
      return to.concat(ar || Array.prototype.slice.call(from));
  }

  var useUpdate = function () {
      var _a = __read(React.useState({}), 2), setState = _a[1];
      return React.useCallback(function () { return setState({}); }, []);
  };

  var useUpdateEffect = function (effect, deps) {
      var isUpdate = React.useRef(false);
      React.useEffect(function () {
          if (!isUpdate.current) {
              isUpdate.current = true;
          }
          else {
              effect();
          }
      }, deps);
  };

  var useDeepEffect = function (effect, deps) {
      var ref = React.useRef(undefined);
      if (!ref.current || !isEqual__default['default'](deps, ref.current)) {
          ref.current = lodash.cloneDeep(deps);
      }
      React.useEffect(effect, ref.current);
  };

  function resolveState(nextState, currentState) {
      if (lodash.isFunction(nextState)) {
          return nextState.length
              ? nextState(currentState)
              : nextState();
      }
      return nextState;
  }

  var useArray = function (initialValue) {
      if (initialValue === void 0) { initialValue = []; }
      var array = React.useRef(resolveState(initialValue));
      var update = useUpdate();
      var actions = React.useMemo(function () {
          var action = {
              set: function (newArray) {
                  array.current = resolveState(newArray, array.current);
                  update();
              },
              reset: function () {
                  action.set(resolveState(initialValue).slice());
              },
              push: function () {
                  var item = [];
                  for (var _i = 0; _i < arguments.length; _i++) {
                      item[_i] = arguments[_i];
                  }
                  if (item.length > 0) {
                      action.set(function (cur) { return cur.concat(item); });
                  }
              },
              updateAt: function (index, item) {
                  action.set(function (cur) {
                      var arr = __spreadArray([], __read(cur), false);
                      arr[index] = item;
                      return arr;
                  });
              },
              insertAt: function (index) {
                  var item = [];
                  for (var _i = 1; _i < arguments.length; _i++) {
                      item[_i - 1] = arguments[_i];
                  }
                  action.set(function (cur) {
                      var arr = __spreadArray([], __read(cur), false);
                      arr.splice.apply(arr, __spreadArray([index, 0], __read(item), false));
                      return arr;
                  });
              },
              removeAt: function (index) {
                  action.set(function (cur) {
                      var arr = __spreadArray([], __read(cur), false);
                      arr.splice(index, 1);
                      return arr;
                  });
              },
              clear: function () {
                  action.set([]);
              },
          };
          return action;
      }, []);
      return [array.current, actions];
  };

  // 会在全局中保存缓存的值集
  var cachedCodes = new Map();
  /**
   * 获取值集
   * @param code 值集编码
   * @param queryCode 值集的查询方法
   * @param isCache 是否缓存值集(默认false)
   */
  var useCode = function (code, queryCode, isCache) {
      var _a = __read(React.useState([]), 2), data = _a[0], setData = _a[1];
      React.useEffect(function () {
          var cacheData = cachedCodes.get(code);
          if (!code)
              return;
          if (!cacheData) {
              queryCode(code).then(function (data) {
                  setData(data);
                  if (isCache) {
                      cachedCodes.set(code, data);
                  }
              });
              return;
          }
          else {
              setData(cacheData);
          }
      }, [code]);
      return data;
  };

  var cachedDataSet = new Map();
  var useDataSet = function (dataSetFactory, deps, cacheKey) {
      return React.useMemo(function () {
          if (cacheKey) {
              var cacheDS = cachedDataSet.get(cacheKey);
              if (!cacheDS) {
                  cacheDS = dataSetFactory();
                  cachedDataSet.set(cacheKey, cacheDS);
              }
              return cacheDS;
          }
          return dataSetFactory();
      }, deps || []);
  };

  var useDataSetEvents = function (dataSet, eventNames, handler, options) {
      var handlerRef = React.useRef();
      handlerRef.current = handler;
      var eventListener = function (event) {
          return handlerRef.current && handlerRef.current(event);
      };
      var handleEventListener = React.useCallback(function (isAdd) {
          var listener = function (eventName) {
              if (isAdd) {
                  dataSet.addEventListener(eventName, eventListener, {
                      capture: options === null || options === void 0 ? void 0 : options.capture,
                      once: options === null || options === void 0 ? void 0 : options.once,
                      passive: options === null || options === void 0 ? void 0 : options.passive,
                  });
              }
              else {
                  dataSet.removeEventListener(eventName, eventListener, {
                      capture: options === null || options === void 0 ? void 0 : options.capture,
                  });
              }
          };
          if (lodash.isArray(eventNames)) {
              eventNames.forEach(function (eventName) {
                  listener(eventName);
              });
          }
          else if (lodash.isString(eventNames)) {
              listener(eventNames);
          }
      }, [dataSet, handler]);
      React.useEffect(function () {
          handleEventListener(true);
          return function () {
              handleEventListener(false);
          };
      }, [eventNames, options === null || options === void 0 ? void 0 : options.capture, options === null || options === void 0 ? void 0 : options.once, options === null || options === void 0 ? void 0 : options.passive]);
  };

  var useDataSetIsSelected = function (dataSet) {
      var _a = __read(React.useState(false), 2), isSelected = _a[0], setIsSelected = _a[1];
      useDataSetEvents(dataSet, ['batchSelect', 'batchUnSelect'], function (_a) {
          var ds = _a.dataSet;
          setIsSelected(ds.selected.length > 0);
      });
      return isSelected;
  };

  /*
  Copyright 2019 今日诗词

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
  */

  // 今日诗词 V2 NPM-SDK 1.0.0
  // 今日诗词API 是一个可以免费调用的诗词接口：https://www.jinrishici.com

  const keyName = "jinrishici-token";

  function load(callback, errHandler) {
      if (window.localStorage && window.localStorage.getItem(keyName)) {
          return commonLoad(callback, errHandler, window.localStorage.getItem(keyName));
      } else {
          return corsLoad(callback, errHandler);
      }
  }

  function corsLoad(callback, errHandler) {
      const newCallBack = function (result) {
          window.localStorage.setItem(keyName, result.token);
          callback(result);
      };
      return sendRequest(newCallBack, errHandler, "https://v2.jinrishici.com/one.json?client=npm-sdk/1.0");
  }

  function commonLoad(callback, errHandler, token) {
      return sendRequest(callback, errHandler, "https://v2.jinrishici.com/one.json?client=npm-sdk/1.0&X-User-Token=" + encodeURIComponent(token));
  }

  function sendRequest(callback, errHandler, apiUrl) {
      var xhr = new XMLHttpRequest();
      xhr.open('get', apiUrl);
      xhr.withCredentials = true;
      xhr.send();
      xhr.onreadystatechange = function () {
          if (xhr.readyState === 4) {
              var data = JSON.parse(xhr.responseText);
              if (data.status === "success") {
                  callback(data);
              } else {
              	if (errHandler) {
              		errHandler(data);
              	} else {
                  	console.error("今日诗词API加载失败，错误原因：" + data.errMessage);
              	}
              }
          }
      };
  }

  var useTodayPoetry = function (intervalTime) {
      var _a = __read(React.useState({
          content: '',
          matchTags: [],
          origin: {},
      }), 2), poetryData = _a[0], setPoetryData = _a[1];
      var _b = __read(React.useState(false), 2), loading = _b[0], setLoading = _b[1];
      React.useEffect(function () {
          var timer;
          init();
          if (lodash.isNumber(intervalTime) && intervalTime > 0) {
              timer = setInterval(function () {
                  init();
              }, intervalTime);
          }
          return function () {
              clearInterval(timer);
          };
      }, [intervalTime]);
      var init = React.useCallback(function () {
          setLoading(true);
          load(function (result) {
              setLoading(false);
              if (result.status === 'success') {
                  setPoetryData(result.data);
              }
          }, function (errData) {
              setLoading(false);
              console.error(errData);
          });
      }, []);
      return {
          content: poetryData.content,
          tags: poetryData.matchTags || [],
          origin: poetryData.origin,
          loading: loading,
          allData: poetryData,
          update: init,
      };
  };

  /**
   *  store 和 dispatch 上下文分离
   * @param reducer
   * @param initialValue
   * @constructor
   */
  var makeStore = function (reducer, initialValue) {
      var storeContext = React.createContext(initialValue);
      var dispatchContext = React.createContext(function () {
      });
      var StoreProvider = function (_a) {
          var children = _a.children;
          var _b = __read(React.useReducer(reducer, initialValue), 2), store = _b[0], dispatch = _b[1];
          return (React__default['default'].createElement(dispatchContext.Provider, { value: dispatch },
              React__default['default'].createElement(storeContext.Provider, { value: store }, children)));
      };
      function useStore() {
          return React.useContext(storeContext);
      }
      function useDispatch() {
          return React.useContext(dispatchContext);
      }
      return [StoreProvider, useStore, useDispatch];
  };

  exports.makeStore = makeStore;
  exports.useArray = useArray;
  exports.useCode = useCode;
  exports.useDataSet = useDataSet;
  exports.useDataSetEvents = useDataSetEvents;
  exports.useDataSetIsSelected = useDataSetIsSelected;
  exports.useDeepEffect = useDeepEffect;
  exports.useEffectOnce = useEffectOnce;
  exports.useMount = useMount;
  exports.useTodayPoetry = useTodayPoetry;
  exports.useUnmount = useUnmount;
  exports.useUpdate = useUpdate;
  exports.useUpdateEffect = useUpdateEffect;

  Object.defineProperty(exports, '__esModule', { value: true });

});
